@@ -47,7 +47,16 @@ void game( bool singleplayer) {
         0,
         WINNER_NONE,
         false,
-        GR_CTT_GRASS
+        GR_CTT_GRASS,
+        { 0 },
+        0,
+        false,
+        { { { 0 } } },
+        0.0,
+        SOUND_MAX,
+        0.0,
+        0.0,
+        0
     };
 
     strcpy( s.game_score_str, format_game( &s));
@@ -59,6 +68,7 @@ void game( bool singleplayer) {
     Uint32 diff;
     Uint32 accumulator = 0;
     bool quit = false;
+    int x, y, z;
 
     if( singleplayer) {
 #ifdef DEBUG
@@ -68,7 +78,15 @@ void game( bool singleplayer) {
     }
 
     game_setup_serve( &s);
-    sound_audience();
+
+    /* smoothen n-gram */
+    for( x = 0; x<NGRAM_STEPS; x++) {
+        for( y = 0; y<NGRAM_STEPS; y++) {
+            for( z = 0; z<NGRAM_STEPS; z++) {
+                s.ngram[x][y][z] = 1;
+            }
+        }
+    }
    
     while( !quit) {
         nt = SDL_GetTicks();
@@ -102,12 +120,12 @@ bool step( GameState* s) {
    
     if( get_phase( s) < 1.0) {
         if( !s->ground.jump) {
-            sound_ground();
+            s->play_sound = SOUND_GROUND;
 
             if( IS_OUT_Y( s->ball.y)) {
                 /* out - responsibilities stay the same */
                 s->status = "out!";
-                sound_out();
+                s->play_sound = SOUND_OUT;
                 s->referee = REFEREE_OUT;
             } else {
                 /* not out - responsibilities change */
@@ -148,13 +166,34 @@ bool step( GameState* s) {
             }
 
             game_setup_serve( s);
-            sound_applause();
+            s->play_sound = SOUND_APPLAUSE;
             SDL_Delay( 500);
             start_fade();
             s->was_stopped = true;
+            s->history_size = 0;
+            s->history_is_locked = 0;
+            s->ngram_prediction = 0.0;
+#ifdef DEBUG
+            printf( "-- game reset --\n");
+#endif
         }
     
         if( IS_OUT_X(s->ball.x)) {
+            if( !s->history_is_locked && s->referee != REFEREE_OUT) {
+                s->history[s->history_size] = (int)(NGRAM_STEPS*s->ball.y/HEIGHT);
+                s->history_size++;
+                if( s->history_size == 3) {
+                    s->ngram[s->history[0]][s->history[1]][s->history[2]] += 10;
+#ifdef DEBUG
+                    printf( "history: %d, %d, %d\n", s->history[0], s->history[1], s->history[2]);
+#endif
+                    s->ngram_prediction = ngram_predictor( s);
+                    s->history[0] = s->history[1];
+                    s->history[1] = s->history[2];
+                    s->history_size--;
+                }
+                s->history_is_locked = true;
+            }
             if( s->ball.move_x <= 0 && IS_NEAR_X( s->player1.x, s->ball.x) && IS_NEAR_Y( s->player1.y, s->ball.y) && s->player1.state && s->referee != REFEREE_OUT) {
                 s->ball.x = GAME_X_MIN;
                 if( s->player1.state == PLAYER_STATE_MAX) {
@@ -165,8 +204,8 @@ bool step( GameState* s) {
                 s->ball.move_y = get_move_y( s, 1);
                 s->player2.responsible = !(s->player1.responsible = 1);
                 s->ball.jump += 1.0-2.0*(s->player1.state<5);
-                sound_applause_stop();
-                sound_racket();
+                s->play_sound = SOUND_RACKET;
+                pan_sample(SOUND_RACKET, 0.4);
             } else if( s->ball.move_x >= 0 && IS_NEAR_X( s->player2.x, s->ball.x) && IS_NEAR_Y( s->player2.y, s->ball.y) && s->player2.state && s->referee != REFEREE_OUT) {
                 s->ball.x = GAME_X_MAX;
                 if( s->player2.state == PLAYER_STATE_MAX) {
@@ -177,10 +216,12 @@ bool step( GameState* s) {
                 s->ball.move_y = get_move_y( s, 2);
                 s->player1.responsible = !(s->player2.responsible = 1);
                 s->ball.jump += 1.0-2.0*(s->player2.state<5);
-                sound_applause_stop();
-                sound_racket();
+                s->play_sound = SOUND_RACKET;
+                pan_sample(SOUND_RACKET, 0.6);
             }
         }
+    } else {
+        s->history_is_locked = false;
     }
 
     /* Update ball_dest for debugging purposes */
@@ -195,6 +236,20 @@ bool step( GameState* s) {
 
     SDL_PollEvent( &e);
     keys = SDL_GetKeyState( NULL);
+    switch(e.type) {
+        case SDL_JOYAXISMOTION:
+            if (e.jaxis.axis == JOYSTICK_Y_AXIS) {
+                s->joystick_y = JOYSTICK_PERCENTIZE(e.jaxis.value);
+            } else if (e.jaxis.axis == JOYSTICK_X_AXIS) {
+                s->joystick_x = JOYSTICK_PERCENTIZE(e.jaxis.value);
+            }
+            break;
+        case SDL_JOYBUTTONUP: case SDL_JOYBUTTONDOWN:
+            if (e.jbutton.button == JOYSTICK_BUTTON_A) {
+                s->joystick_a = (e.jbutton.state == SDL_PRESSED);
+            }
+            break;
+    }
 
     if( keys['c'] && s->time%50==0) {
         s->court_type++;
@@ -205,20 +260,43 @@ bool step( GameState* s) {
 
     if( !is_fading() && !s->is_over) {
         if( s->player1.type == PLAYER_TYPE_HUMAN) {
-           input_human( &s->player1, keys['w'], keys['s'], keys['d']);
+           input_human( &s->player1,
+                   keys['w'] || keys[SDLK_UP] || s->joystick_y < -JOYSTICK_TRESHOLD,
+                   keys['s'] || keys[SDLK_DOWN] || s->joystick_y > JOYSTICK_TRESHOLD,
+                   keys['d'] || keys[SDLK_SPACE] || keys[SDLK_LCTRL] || s->joystick_a,
+                   s);
         } else {
-            input_ai( &s->player1, &s->ball, &s->player2);
+            input_ai( &s->player1, &s->ball, &s->player2, s);
         }
  
         if( s->player2.type == PLAYER_TYPE_HUMAN) {
-            input_human( &s->player2, keys['o'], keys['l'], keys['k']);
+            input_human( &s->player2, keys['o'], keys['l'], keys['k'], s);
         } else {
-            input_ai( &s->player2, &s->ball, &s->player1);
+            input_ai( &s->player2, &s->ball, &s->player1, s);
         }
     }
     
     if( keys['f']) SDL_WM_ToggleFullScreen( screen);
     if( keys['y']) SDL_SaveBMP( screen, "screenshot.bmp");
+
+    if( keys['p']) {
+        while( keys['p']) {
+            SDL_PollEvent( &e);
+            keys = SDL_GetKeyState( NULL);
+            SDL_Delay( 10);
+        }
+        while( keys['p'] == 0) {
+            SDL_PollEvent( &e);
+            keys = SDL_GetKeyState( NULL);
+            SDL_Delay( 10);
+        }
+        while( keys['p']) {
+            SDL_PollEvent( &e);
+            keys = SDL_GetKeyState( NULL);
+            SDL_Delay( 10);
+        }
+        s->was_stopped = true;
+    }
      
     if( keys[SDLK_ESCAPE] || keys['q']) return true;
     
@@ -230,6 +308,10 @@ bool step( GameState* s) {
 }
 
 void render( GameState* s) {
+    if (s->play_sound != SOUND_MAX) {
+        play_sample(s->play_sound);
+        s->play_sound = SOUND_MAX;
+    }
     if( s->winner != WINNER_NONE) {
         if( !s->is_over) {
             start_fade();
@@ -352,13 +434,22 @@ float get_move_y( GameState* s, unsigned char player) {
     return (y_len*move_x)/(x_len);
 }
 
-void input_human( Player* player, bool up, bool down, bool hit) {
+void input_human( Player* player, bool up, bool down, bool hit, GameState* s) {
+    int diff = PLAYER_MOVE_Y;
+
+    if (fabsf(s->joystick_y) > JOYSTICK_TRESHOLD) {
+        diff = PLAYER_MOVE_Y*fabsf(s->joystick_y)/40.0;
+        if (diff > PLAYER_MOVE_Y) {
+            diff = PLAYER_MOVE_Y;
+        }
+    }
+
     if( up) {
-        player->y -= 6;
+        player->y -= diff;
     }
 
     if( down) {
-        player->y += 6;
+        player->y += diff;
     }
 
     if( hit) {
@@ -371,15 +462,15 @@ void input_human( Player* player, bool up, bool down, bool hit) {
     }
 }
 
-void input_ai( Player* player, Ball* ball, Player* opponent) {
-    float fact = 1.5;
+void input_ai( Player* player, Ball* ball, Player* opponent, GameState* s) {
+    float fact = 1.7;
     float target;
 
     if( fabsf( player->y - ball->y) > RACKET_Y_MID*5) {
-        fact = 4;
+        fact = 3.5;
     }
 
-    target = GAME_Y_MID + (opponent->ball_dest - GAME_Y_MID)/10;
+    target = GAME_Y_MID + (opponent->ball_dest - GAME_Y_MID)/5;
 
     if( player->responsible) {
         if( player->desire == DESIRE_NORMAL && !IS_NEAR_Y_AI( player->y, ball->y)) {
@@ -394,6 +485,17 @@ void input_ai( Player* player, Ball* ball, Player* opponent) {
             player->state = PLAYER_STATE_MAX;
         }
     } else if( ball->move_x == 0) {
+        if( player->desire == DESIRE_NORMAL && !IS_NEAR_Y_AI( player->y, target)) {
+            if( player->y < target) {
+                player->y += fmin( fact, (target-player->y)/40.0);
+            } else if( player->y > target) {
+                player->y -= fmin( fact, (player->y-target)/40.0);
+            }
+        }
+    } else if( s->ngram_prediction > 0.0) {
+        target = s->ngram_prediction*((float)HEIGHT)/((float)(NGRAM_STEPS));
+        target = GAME_Y_MID + (target-GAME_Y_MID)*1.5;
+
         if( player->desire == DESIRE_NORMAL && !IS_NEAR_Y_AI( player->y, target)) {
             if( player->y < target) {
                 player->y += fmin( fact, (target-player->y)/40.0);
@@ -429,6 +531,32 @@ void game_setup_serve( GameState* s) {
     s->player2.responsible = !(s->player1.responsible);
 }
 
+float ngram_predictor( GameState* s) {
+    unsigned int count = 0;
+    unsigned long sum = 0;
+    int x, y, z;
+    float result;
+
+    if( s->history_size < 3) {
+        return 0.0;
+    }
+
+    x = s->history[1];
+    y = s->history[2];
+
+    for( z = 0; z<NGRAM_STEPS; z++) {
+        count += s->ngram[x][y][z];
+        sum += z * s->ngram[x][y][z];
+    }
+
+    result = ((float)(sum))/((float)(count));
+#ifdef DEBUG
+    printf( "predicting next = %.2f\n", result);
+#endif
+
+    return result;
+}
+
 void score_game( GameState* s, bool player1_scored) {
     Player* winner = (player1_scored)?(&(s->player1)):(&(s->player2));
     Player* loser = (player1_scored)?(&(s->player2)):(&(s->player1));
